@@ -7,6 +7,7 @@ import functools
 import inspect
 import ipaddress
 import pathlib
+import uuid
 from operator import attrgetter
 from typing import (
     Any,
@@ -18,6 +19,7 @@ from typing import (
     Type,
     Tuple,
     TypeVar,
+    Iterable,
 )
 
 import typic
@@ -71,6 +73,7 @@ __all__ = (
     "issubclass",
     "istypeddict",
     "istypedtuple",
+    "isuuidtype",
     "iswriteonly",
     "should_unwrap",
 )
@@ -336,9 +339,41 @@ def isdatetype(obj: Type[ObjectT]) -> bool:
     return _issubclass(util.origin(obj), (datetime.datetime, datetime.date))
 
 
+@functools.lru_cache(maxsize=None)
+def isuuidtype(obj: Type[ObjectT]) -> bool:
+    """Test whether this annotation is a a date/datetime object.
+
+    Parameters
+    ----------
+    obj
+
+    Examples
+    --------
+
+    >>> import typic
+    >>> import uuid
+    >>> from typing import NewType
+    >>> typic.isuuidtype(uuid.UUID)
+    True
+    >>> class MyUUID(uuid.UUID): ...
+    ...
+    >>> typic.isuuidtype(MyUUID)
+    True
+    >>> typic.isuuidtype(NewType("Foo", uuid.UUID))
+    True
+    """
+    return _issubclass(util.origin(obj), uuid.UUID)
+
+
 _COLLECTIONS = {list, set, tuple, frozenset, dict, str, bytes}
 
 
+@functools.lru_cache(maxsize=None)
+def isiterabletype(obj: Type[ObjectT]):
+    obj = util.origin(obj)
+    return _issubclass(obj, Iterable)
+
+
 @functools.lru_cache(maxsize=None)
 def iscollectiontype(obj: Type[ObjectT]):
     """Test whether this annotation is a subclass of :py:class:`typing.Collection`.
