@@ -32,6 +32,7 @@ from typic.checks import (
     isnamedtuple,
     should_unwrap,
 )
+from typic.compat import ForwardRef
 from typic.types import dsn, email, frozendict, path, secret, url
 from typic.util import (
     origin,
@@ -48,7 +49,14 @@ from .array import (
     SetContraints,
     TupleContraints,
 )
-from .common import MultiConstraints, TypeConstraints, EnumConstraints, VT
+from .common import (
+    MultiConstraints,
+    TypeConstraints,
+    EnumConstraints,
+    VT,
+    DelayedConstraints,
+    ForwardDelayedConstraints,
+)
 from .mapping import (
     MappingConstraints,
     DictConstraints,
@@ -67,9 +75,11 @@ from .text import BytesConstraints, StrConstraints
 ConstraintsT = Union[
     BytesConstraints,
     DecimalContraints,
+    DelayedConstraints,
     DictConstraints,
     EnumConstraints,
     FloatContraints,
+    ForwardDelayedConstraints,
     FrozenSetConstraints,
     IntContraints,
     ListContraints,
@@ -95,7 +105,9 @@ ArrayConstraintsT = Union[
 ]
 
 
-def _resolve_args(*args, nullable: bool = False) -> Optional[ConstraintsT]:
+def _resolve_args(
+    *args, cls: Type = None, nullable: bool = False
+) -> Optional[ConstraintsT]:
     largs: List = [*args]
     items: List[ConstraintsT] = []
 
@@ -104,20 +116,20 @@ def _resolve_args(*args, nullable: bool = False) -> Optional[ConstraintsT]:
         if arg in {Any, Ellipsis}:
             continue
         if origin(arg) is Union:
-            c = _from_union(arg, nullable=nullable)
+            c = _from_union(arg, cls=cls, nullable=nullable)
             if isinstance(c, MultiConstraints):
                 items.extend(c.constraints)
             else:
                 items.append(c)
             continue
-        items.append(get_constraints(arg, nullable=nullable))
+        items.append(_maybe_get_delayed(arg, cls=cls, nullable=nullable))
     if len(items) == 1:
         return items[0]
     return MultiConstraints((*items,))  # type: ignore
 
 
 def _from_array_type(
-    t: Type[Array], *, nullable: bool = False, name: str = None
+    t: Type[Array], *, nullable: bool = False, name: str = None, cls: Type = None
 ) -> ArrayConstraintsT:
     args = get_args(t)
     constr_class = cast(
@@ -126,13 +138,13 @@ def _from_array_type(
     # If we don't have args, then return a naive constraint
     if not args:
         return constr_class(nullable=nullable, name=name)
-    items = _resolve_args(*args, nullable=nullable)
+    items = _resolve_args(*args, cls=cls, nullable=nullable)
 
     return constr_class(nullable=nullable, values=items, name=name)
 
 
 def _from_mapping_type(
-    t: Type[Mapping], *, nullable: bool = False, name: str = None
+    t: Type[Mapping], *, nullable: bool = False, name: str = None, cls: Type = None
 ) -> Union[MappingConstraints, DictConstraints]:
     if isbuiltintype(t):
         return DictConstraints(nullable=nullable, name=name)
@@ -145,7 +157,10 @@ def _from_mapping_type(
     if not args:
         return constr_class(nullable=nullable, name=name)
     key_arg, value_arg = args
-    key_items, value_items = _resolve_args(key_arg), _resolve_args(value_arg)
+    key_items, value_items = (
+        _resolve_args(key_arg, cls=cls),
+        _resolve_args(value_arg, cls=cls),
+    )
     return constr_class(
         keys=key_items, values=value_items, nullable=nullable, name=name
     )
@@ -167,7 +182,7 @@ _SIMPLE_CONSTRAINTS = TypeMap(
 
 
 def _from_simple_type(
-    t: Type[SimpleT], *, nullable: bool = False, name: str = None
+    t: Type[SimpleT], *, nullable: bool = False, name: str = None, cls: Type = None
 ) -> SimpleConstraintsT:
     constr_class = cast(
         Type[SimpleConstraintsT], _SIMPLE_CONSTRAINTS.get_by_parent(origin(t))
@@ -175,46 +190,50 @@ def _from_simple_type(
     return constr_class(nullable=nullable, name=name)
 
 
-def _resolve_params(**param: inspect.Parameter,) -> Mapping[str, ConstraintsT]:
+def _resolve_params(
+    cls: Type, **param: inspect.Parameter,
+) -> Mapping[str, ConstraintsT]:
     items: Dict[str, ConstraintsT] = {}
-
     while param:
         name, p = param.popitem()
         anno = p.annotation
-        nullable = p.default is None or isoptionaltype(anno)
+        nullable = p.default in (None, Ellipsis) or isoptionaltype(anno)
         if anno in {Any, Ellipsis, p.empty}:
             continue
         if origin(anno) is Union:
-            items[name] = _from_union(anno, nullable=nullable, name=name)
+            items[name] = _from_union(anno, nullable=nullable, name=name, cls=cls)
             continue
-        items[name] = get_constraints(anno, nullable=nullable, name=name)
+        else:
+            items[name] = _maybe_get_delayed(
+                anno, nullable=nullable, name=name, cls=cls
+            )
     return items
 
 
 def _from_strict_type(
-    t: Type[VT], *, nullable: bool = False, name: str = None
+    t: Type[VT], *, nullable: bool = False, name: str = None, cls: Type = None
 ) -> TypeConstraints:
     return TypeConstraints(t, nullable=nullable, name=name)
 
 
 def _from_enum_type(
-    t: Type[enum.Enum], *, nullable: bool = False, name: str = None
+    t: Type[enum.Enum], *, nullable: bool = False, name: str = None, cls: Type = None
 ) -> EnumConstraints:
     return EnumConstraints(t, nullable=nullable, name=name)
 
 
 def _from_union(
-    t: Type[VT], *, nullable: bool = False, name: str = None
+    t: Type[VT], *, nullable: bool = False, name: str = None, cls: Type = None
 ) -> ConstraintsT:
     _nullable: bool = isoptionaltype(t)
     nullable = nullable or _nullable
     _args = get_args(t)[:-1] if _nullable else get_args(t)
     if len(_args) == 1:
-        return get_constraints(_args[0], nullable=nullable, name=name)
+        return _maybe_get_delayed(_args[0], nullable=nullable, name=name, cls=cls)
     return MultiConstraints(
         (
             *(
-                get_constraints(a, nullable=nullable)  # type: ignore
+                _maybe_get_delayed(a, nullable=nullable, cls=cls)  # type: ignore
                 for a in _args
             ),
         ),
@@ -223,7 +242,7 @@ def _from_union(
 
 
 def _from_class(
-    t: Type[VT], *, nullable: bool = False, name: str = None
+    t: Type[VT], *, nullable: bool = False, name: str = None, cls: Type = None
 ) -> Union[ObjectConstraints, TypeConstraints, MappingConstraints]:
     if not istypeddict(t) and not isnamedtuple(t) and isbuiltinsubtype(t):
         return _from_strict_type(t, nullable=nullable, name=name)
@@ -240,7 +259,7 @@ def _from_class(
     name = name or get_name(t)
     items: Optional[
         frozendict.FrozenDict[Hashable, ConstraintsT]
-    ] = frozendict.FrozenDict(_resolve_params(**params)) or None
+    ] = frozendict.FrozenDict(_resolve_params(t, **params)) or None
     required = frozenset(
         (
             pname
@@ -306,9 +325,29 @@ _CONSTRAINT_BUILDER_HANDLERS = TypeMap(
 )
 
 
+@functools.lru_cache(maxsize=None)
+def _maybe_get_delayed(
+    t: Type[VT], *, nullable: bool = False, name: str = None, cls: Type = None
+):
+    if t.__class__ is ForwardRef:
+        return ForwardDelayedConstraints(
+            t,  # type: ignore
+            cls.__module__,
+            localns=getattr(cls, "__dict__", {}).copy(),
+            nullable=nullable,
+            name=name,  # type: ignore
+            factory=get_constraints,
+        )
+    elif t is cls:
+        return DelayedConstraints(
+            t, nullable=nullable, name=name, factory=get_constraints  # type: ignore
+        )
+    return get_constraints(t, nullable=nullable, name=name, cls=cls)
+
+
 @functools.lru_cache(maxsize=None)
 def get_constraints(
-    t: Type[VT], *, nullable: bool = False, name: str = None
+    t: Type[VT], *, nullable: bool = False, name: str = None, cls: Type = None
 ) -> ConstraintsT:
     while should_unwrap(t):
         nullable = nullable or isoptionaltype(t)
@@ -324,5 +363,5 @@ def get_constraints(
         handler = _from_class
     else:
         handler = _CONSTRAINT_BUILDER_HANDLERS.get_by_parent(origin(t), _from_class)  # type: ignore
-    c = handler(t, nullable=nullable, name=name)  # type: ignore
+    c = handler(t, nullable=nullable, name=name, cls=cls)  # type: ignore
     return c
