@@ -2,6 +2,7 @@ from typing import Callable, Any, AnyStr
 
 dumps: Callable[..., AnyStr]
 loads: Callable[..., Any]
+NATIVE_JSON = False
 
 
 try:
@@ -11,7 +12,12 @@ try:
     loads = ujson.loads
 
 except (ImportError, ModuleNotFoundError):  # pragma: nocover
+    NATIVE_JSON = True
     import json
 
     dumps = json.dumps  # type: ignore
     loads = json.loads
+
+
+def using_stdlib() -> bool:
+    return NATIVE_JSON
