@@ -8,6 +8,7 @@ import enum
 import ipaddress
 import pathlib
 import re
+import reprlib
 import uuid
 from typing import (
     ClassVar,
@@ -30,7 +31,7 @@ import pendulum
 from typic.ext.json import dumps
 from typic.serde.common import SerdeFlags
 from typic.serde.resolver import resolver
-from typic.util import filtered_repr, cached_property, TypeMap, ReprT, apply_slots
+from typic.util import filtered_repr, cached_property, TypeMap, ReprT, slotted
 from typic.types import dsn, email, frozendict, path, secret, url
 from .compat import fastjsonschema
 
@@ -85,7 +86,7 @@ class _Serializable:
         )
 
 
-@apply_slots
+@slotted
 @dataclasses.dataclass(frozen=True)
 class Ref(_Serializable):
     """A JSON Schema ref (pointer).
@@ -118,7 +119,7 @@ class StringFormat(str, enum.Enum):
     IPV6 = "ipv6"
 
 
-@apply_slots
+@slotted
 @dataclasses.dataclass(frozen=True, repr=False)
 class BaseSchemaField(_Serializable):
     """The base JSON Schema Field."""
@@ -136,7 +137,7 @@ class BaseSchemaField(_Serializable):
     writeOnly: Optional[bool] = None
     extensions: Optional[Tuple[frozendict.FrozenDict[str, Any], ...]] = None
 
-    __repr = cached_property(filtered_repr)
+    __repr = cached_property(reprlib.recursive_repr()(filtered_repr))
 
     def __repr__(self) -> str:  # pragma: nocover
         return self.__repr
@@ -186,7 +187,7 @@ class BaseSchemaField(_Serializable):
         return copy.deepcopy(self)
 
 
-@apply_slots
+@slotted
 @dataclasses.dataclass(frozen=True, repr=False)
 class UndeclaredSchemaField(BaseSchemaField):
     """A sentinel object for generating an empty schema."""
@@ -194,7 +195,7 @@ class UndeclaredSchemaField(BaseSchemaField):
     pass
 
 
-@apply_slots
+@slotted
 @dataclasses.dataclass(frozen=True, repr=False)
 class MultiSchemaField(BaseSchemaField):
     """A schema field which supports multiple types."""
@@ -204,13 +205,13 @@ class MultiSchemaField(BaseSchemaField):
     oneOf: Optional[Tuple["SchemaFieldT", ...]] = None
 
 
-@apply_slots
+@slotted
 @dataclasses.dataclass(frozen=True, repr=False)
 class NullSchemaField(BaseSchemaField):
     type = SchemaType.NULL
 
 
-@apply_slots
+@slotted
 @dataclasses.dataclass(frozen=True, repr=False)
 class StrSchemaField(BaseSchemaField):
     """A JSON Schema Field for the `string` type.
@@ -230,7 +231,7 @@ class StrSchemaField(BaseSchemaField):
 Number = Union[int, float, decimal.Decimal]
 
 
-@apply_slots
+@slotted
 @dataclasses.dataclass(frozen=True, repr=False)
 class IntSchemaField(BaseSchemaField):
     """A JSON Schema Field for the `integer` type.
@@ -248,7 +249,7 @@ class IntSchemaField(BaseSchemaField):
     exclusiveMinimum: Optional[Number] = None
 
 
-@apply_slots
+@slotted
 @dataclasses.dataclass(frozen=True, repr=False)
 class NumberSchemaField(IntSchemaField):
     """A JSON Schema Field for the `number` type.
@@ -261,7 +262,7 @@ class NumberSchemaField(IntSchemaField):
     type = SchemaType.NUM
 
 
-@apply_slots
+@slotted
 @dataclasses.dataclass(frozen=True, repr=False)
 class BooleanSchemaField(BaseSchemaField):
     """A JSON Schema Field for the `boolean` type.
@@ -274,7 +275,7 @@ class BooleanSchemaField(BaseSchemaField):
     type = SchemaType.BOOL
 
 
-@apply_slots
+@slotted
 @dataclasses.dataclass(frozen=True, repr=False)
 class ObjectSchemaField(BaseSchemaField):
     """A JSON Schema Field for the `object` type.
@@ -296,7 +297,7 @@ class ObjectSchemaField(BaseSchemaField):
     definitions: Optional[frozendict.FrozenDict[str, Any]] = None
 
 
-@apply_slots
+@slotted
 @dataclasses.dataclass(frozen=True, repr=False)
 class ArraySchemaField(BaseSchemaField):
     """A JSON Schema Field for the `array` type.
@@ -325,6 +326,7 @@ SchemaFieldT = Union[
     MultiSchemaField,
     UndeclaredSchemaField,
     NullSchemaField,
+    Ref,
 ]
 """A type-alias for the defined JSON Schema Fields."""
 
