@@ -3,6 +3,7 @@ import functools
 import inspect
 import pathlib
 import re
+import uuid
 from collections import deque, defaultdict, abc
 from operator import attrgetter
 from typing import (
@@ -12,7 +13,6 @@ from typing import (
     Callable,
     Type,
     Dict,
-    Collection,
     Tuple,
     List,
     Pattern,
@@ -34,7 +34,15 @@ from typic.util import (
     get_unique_name,
 )
 from typic.common import DEFAULT_ENCODING, VAR_POSITIONAL, VAR_KEYWORD, ObjectT
-from .common import DeserializerT, DeserializerRegistryT, SerdeConfig, Annotation
+from .common import (
+    DeserializerT,
+    DeserializerRegistryT,
+    SerdeConfig,
+    Annotation,
+    DelayedAnnotation,
+    ForwardDelayedAnnotation,
+    AnnotationT,
+)
 
 if TYPE_CHECKING:  # pragma: nocover
     from .resolver import Resolver
@@ -112,25 +120,42 @@ class DesFactory:
         self.__USER_DESS.appendleft((check, deserializer))
 
     def _set_checks(self, func: gen.Block, anno_name: str, annotation: "Annotation"):
-        _checks = []
         _ctx = {}
+        # run a safe eval if input is text and anno isn't
+        if issubclass(annotation.resolved_origin, (str, bytes)):
+            self._add_vtype(func)
+        else:
+            self._add_eval(func)
+        # Equality checks for defaults and optionals
+        custom_equality = hasattr(annotation.resolved_origin, "equals")
+        if custom_equality and (annotation.optional or annotation.has_default):
+            func.l(f"custom_equality = hasattr({self.VNAME}, 'equals')")
+        null = ""
         if annotation.optional:
-            _checks.append(f"{self.VNAME} is None")
-        if annotation.has_default:
-            if hasattr(annotation.resolved_origin, "equals"):
-                eq = (
-                    f"({self.VNAME}.equals(__default) "
-                    f"if hasattr({self.VNAME}, 'equals') "
-                    f"else {self.VNAME} == __default)"
+            null = f"{self.VNAME} in {self.resolver.OPTIONALS}"
+            if custom_equality:
+                null = (
+                    f"(any({self.VNAME}.equals(o) for o in {self.resolver.OPTIONALS}) "
+                    "if custom_equality "
+                    f"else {null})"
                 )
-            else:
-                eq = f"{self.VNAME} == __default"
-            _checks.append(f"({self.VTYPE} is {anno_name} and {eq})")
+        eq = ""
+        if (
+            annotation.has_default
+            and annotation.parameter.default not in self.resolver.OPTIONALS
+        ):
+            eq = f"{self.VNAME} == __default"
+            if custom_equality:
+                if hasattr(annotation.parameter.default, "equals"):
+                    eq = f"__default.equals({self.VNAME})"
+                eq = f"{self.VNAME}.equals(__default) if custom_equality else {eq}"
             _ctx["__default"] = annotation.parameter.default
-        if _checks:
-            check = " or ".join(_checks)
-            func.l(f"if {check}:", **_ctx)
-            with func.b() as b:
+        if eq or null:
+            # Add a type-check for anything that isn't a builtin.
+            if eq and not checks.isbuiltintype(annotation.resolved_origin):
+                eq = f"{self.VTYPE} is {anno_name} and {eq}"
+            check = " or ".join(c for c in (null, eq) if c)
+            with func.b(f"if {check}:", **_ctx) as b:  # type: ignore
                 b.l(f"return {self.VNAME}")
 
     @staticmethod
@@ -190,6 +215,25 @@ class DesFactory:
                 line = f"{line}.date()"
             b.l(line, dateparse=dateparse)
 
+    def _build_uuid_des(
+        self, func: gen.Block, anno_name: str, annotation: "Annotation"
+    ):
+        self._add_type_check(func, anno_name)
+        with func.b(f"if issubclass({self.VTYPE}, UUID):", UUID=uuid.UUID) as b:
+            b.l(f"{self.VNAME} = {anno_name}(int={self.VNAME}.int)")
+
+        with func.b(f"elif isinstance({self.VNAME}, str):") as b:
+            b.l(f"{self.VNAME} = {anno_name}({self.VNAME})")
+
+        with func.b(f"elif isinstance({self.VNAME}, bytes):") as b:
+            b.l(f"{self.VNAME} = {anno_name}(bytes={self.VNAME})")
+
+        with func.b(f"elif isinstance({self.VNAME}, int):") as b:
+            b.l(f"{self.VNAME} = {anno_name}(int={self.VNAME})")
+
+        with func.b(f"elif isinstance({self.VNAME}, tuple):") as b:
+            b.l(f"{self.VNAME} = {anno_name}(fields={self.VNAME})")
+
     def _add_eval(self, func: gen.Block):
         func.l(
             f"_, {self.VNAME} = __eval({self.VNAME}) "
@@ -206,43 +250,46 @@ class DesFactory:
     def _add_vtype(self, func: gen.Block):
         func.l(f"{self.VTYPE} = {self.VNAME}.__class__")
 
-    def _get_default_factory(self, annotation: "Annotation"):
+    def _get_default_factory(self, annotation: "AnnotationT"):
         factory: Union[Type, Callable[..., Any], None] = None
-
-        if annotation.args:
-            factory_anno = self.resolver.annotation(annotation.args[-1])
-            factory = factory_anno.resolved_origin
-            if issubclass(factory_anno.resolved_origin, defaultdict):
+        args: Tuple = annotation.args if isinstance(annotation, Annotation) else tuple()
+        if args:
+            factory_anno = self.resolver.annotation(args[-1])
+            if isinstance(factory_anno, ForwardDelayedAnnotation):
+                return factory
+            elif isinstance(factory_anno, DelayedAnnotation):
+                use = factory_anno.type
+                raw = use
+            else:
+                use = factory_anno.resolved_origin
+                raw = factory_anno.un_resolved
+            factory = use
+            if issubclass(use, defaultdict):
                 factory_nested = self._get_default_factory(factory_anno)
 
                 def factory():
                     return defaultdict(factory_nested)
 
-                factory.__qualname__ = f"factory({repr(factory_anno.un_resolved)})"  # type: ignore
+                factory.__qualname__ = f"factory({repr(raw)})"  # type: ignore
 
-            if not checks.isbuiltinsubtype(factory_anno.resolved_origin):  # type: ignore
+            if not checks.isbuiltinsubtype(use):  # type: ignore
 
                 params: Mapping[str, inspect.Parameter] = cached_signature(
-                    factory_anno.resolved_origin
+                    use
                 ).parameters
                 if not any(p.default is p.empty for p in params.values()):
 
-                    def factory(*, __origin=factory_anno.resolved_origin):
+                    def factory(*, __origin=use):
                         return __origin()
 
-                    factory.__qualname__ = f"factory({repr(factory_anno.un_resolved)})"  # type: ignore
+                    factory.__qualname__ = f"factory({repr(raw)})"  # type: ignore
 
         return factory
 
-    def _build_builtin_des(
+    def _build_text_des(
         self, func: gen.Block, anno_name: str, annotation: "Annotation",
     ):
         origin = annotation.resolved_origin
-        # We should try and evaluate inputs for anything that isn't a subclass of str.
-        if issubclass(origin, (Collection, bool, int)) and not issubclass(
-            origin, (str, bytes)
-        ):
-            self._add_eval(func)
         # Encode for bytes
         if issubclass(origin, bytes):
             with func.b(f"if isinstance({self.VNAME}, str):") as b:
@@ -254,15 +301,21 @@ class DesFactory:
         elif issubclass(origin, str):
             with func.b(f"if isinstance({self.VNAME}, (bytes, bytearray)):") as b:
                 b.l(f"{self.VNAME} = {self.VNAME}.decode({DEFAULT_ENCODING!r})")
-        if issubclass(origin, defaultdict):
-            func.namespace[anno_name] = functools.partial(defaultdict, None)
-        # Translate fields for a mapping
-        if issubclass(origin, Mapping) and annotation.serde.fields_in:
-            line = f"{anno_name}({{fields_in[x]: y for x, y in {self.VNAME}.items()}})"
-        # Coerce to the target type.
+        func.l(f"{self.VNAME} = {anno_name}({self.VNAME})")
+
+    def _build_builtin_des(
+        self, func: gen.Block, anno_name: str, annotation: "Annotation",
+    ):
+        origin = annotation.resolved_origin
+        if issubclass(origin, (str, bytes)):
+            self._build_text_des(func, anno_name, annotation)
+        elif checks.ismappingtype(origin):
+            self._build_mapping_des(func, anno_name, annotation)
+        elif checks.iscollectiontype(origin):
+            self._build_collection_des(func, anno_name, annotation)
+        # bool, int, float...
         else:
-            line = f"{anno_name}({self.VNAME})"
-        func.l(f"{self.VNAME} = {line}")
+            func.l(f"{self.VNAME} = {anno_name}({self.VNAME})")
 
     def _build_pattern_des(self, func: gen.Block, anno_name: str):
         func.l(
@@ -274,7 +327,6 @@ class DesFactory:
 
     def _build_fromdict_des(self, func: gen.Block, anno_name: str):
         self._add_type_check(func, anno_name)
-        self._add_eval(func)
         func.l(f"{self.VNAME} = {anno_name}.from_dict({self.VNAME})")
 
     def _build_typeddict_des(
@@ -284,28 +336,43 @@ class DesFactory:
         annotation: "Annotation",
         *,
         total: bool = True,
-        eval: bool = True,
+        namespace: Type = None,
     ):
-        if eval:
-            self._add_eval(func)
 
-        fields_deser = {
-            x: self.resolver._resolve_from_annotation(y).transmute
-            for x, y in annotation.serde.fields.items()
-        }
-        x = "fields_in[x]"
-        y = f"fields_deser[x]({self.VNAME}[x])" if fields_deser else f"{self.VNAME}[x]"
-        line = f"{{{x}: {y} for x in fields_in.keys()"
-        tail = "}" if total else f"& {self.VNAME}.keys()}}"
-        func.l(f"{self.VNAME} = {anno_name}(**{line}{tail})", fields_deser=fields_deser)
+        with func.b(f"if issubclass({self.VTYPE}, Mapping):", Mapping=abc.Mapping) as b:
+            fields_deser = {
+                x: self.resolver._resolve_from_annotation(
+                    y, _namespace=namespace
+                ).transmute
+                for x, y in annotation.serde.fields.items()
+            }
+            x = "fields_in[x]"
+            y = (
+                f"fields_deser[x]({self.VNAME}[x])"
+                if fields_deser
+                else f"{self.VNAME}[x]"
+            )
+            line = f"{{{x}: {y} for x in fields_in.keys()"
+            tail = "}" if total else f"& {self.VNAME}.keys()}}"
+            b.l(
+                f"{self.VNAME} = {anno_name}(**{line}{tail})", fields_deser=fields_deser
+            )
+        with func.b("else:") as b:
+            b.l(
+                f"{self.VNAME} = translate({self.VNAME}, {anno_name})",
+                translate=self.resolver.translate,
+            )
 
     def _build_typedtuple_des(
-        self, func: gen.Block, anno_name: str, annotation: "Annotation"
+        self,
+        func: gen.Block,
+        anno_name: str,
+        annotation: "Annotation",
+        namespace: Type = None,
     ):
-        self._add_eval(func)
         with func.b(f"if issubclass({self.VTYPE}, Mapping):", Mapping=abc.Mapping) as b:
             if annotation.serde.fields:
-                self._build_typeddict_des(b, anno_name, annotation, eval=False)
+                self._build_typeddict_des(b, anno_name, annotation, namespace=namespace)
             else:
                 b.l(f"{self.VNAME} = {anno_name}(**{self.VNAME})",)
         with func.b(
@@ -318,23 +385,37 @@ class DesFactory:
                 )
             else:
                 b.l(f"{self.VNAME} = {anno_name}(*{self.VNAME})",)
+        with func.b("else:") as b:
+            b.l(
+                f"{self.VNAME} = translate({self.VNAME}, {anno_name})",
+                translate=self.resolver.translate,
+            )
 
     def _build_mapping_des(
-        self, func: gen.Block, anno_name: str, annotation: "Annotation",
+        self,
+        func: gen.Block,
+        anno_name: str,
+        annotation: "Annotation",
+        namespace: Type = None,
     ):
         key_des, item_des = None, None
         args = annotation.args
         if args:
             args = cast(Tuple[Type, Type], args)
             key_type, item_type = args
-            key_des = self.resolver.resolve(key_type, flags=annotation.serde.flags)
-            item_des = self.resolver.resolve(item_type, flags=annotation.serde.flags)
+            key_des = self.resolver.resolve(
+                key_type, flags=annotation.serde.flags, namespace=namespace
+            )
+            item_des = self.resolver.resolve(
+                item_type, flags=annotation.serde.flags, namespace=namespace
+            )
         if issubclass(annotation.resolved_origin, defaultdict):
             factory = self._get_default_factory(annotation)
             func.namespace[anno_name] = functools.partial(defaultdict, factory)
         kd_name = f"{anno_name}_key_des"
         it_name = f"{anno_name}_item_des"
-        line = f"{anno_name}({self.VNAME})"
+        iterate = f"iterate({self.VNAME})"
+        line = f"{anno_name}({iterate})"
         if args or annotation.serde.fields_in:
             x, y = "x", "y"
             # If there are args & field mapping, get the correct field name
@@ -348,31 +429,53 @@ class DesFactory:
             elif args:
                 x = f"{kd_name}(x)"
                 y = f"{it_name}(y)"
-            # Write the line.
-            line = f"{anno_name}({{{x}: {y} for x, y in {self.VNAME}.items()}})"
-        self._add_eval(func)
+            line = f"{anno_name}({{{x}: {y} for x, y in {iterate}}})"
+        # If we don't have nested annotations, we can short-circuit on valid inputs
+        else:
+            self._add_type_check(func, anno_name)
+        # Write the lines.
         func.l(
             f"{self.VNAME} = {line}",
             level=None,
-            **{kd_name: key_des, it_name: item_des},
+            **{
+                kd_name: key_des,
+                it_name: item_des,
+                "Mapping": abc.Mapping,
+                "iterate": self.resolver.iterate,
+            },
         )
 
     def _build_collection_des(
-        self, func: gen.Block, anno_name: str, annotation: "Annotation"
+        self,
+        func: gen.Block,
+        anno_name: str,
+        annotation: "Annotation",
+        namespace: Type = None,
     ):
         item_des = None
         it_name = f"{anno_name}_item_des"
-        line = f"{self.VNAME} = {anno_name}({self.VNAME})"
+        iterate = f"iterate({self.VNAME}, values=True)"
+        line = f"{self.VNAME} = {anno_name}({iterate})"
         if annotation.args:
             item_type = annotation.args[0]
-            item_des = self.resolver.resolve(item_type, flags=annotation.serde.flags)
+            item_des = self.resolver.resolve(
+                item_type, flags=annotation.serde.flags, namespace=namespace
+            )
             line = (
                 f"{self.VNAME} = "
-                f"{anno_name}({it_name}(x) for x in parent({self.VNAME}))"
+                f"{anno_name}({it_name}(x) for x in parent({iterate}))"
             )
-
-        self._add_eval(func)
-        func.l(line, level=None, **{it_name: item_des})
+        else:
+            self._add_type_check(func, anno_name)
+        func.l(
+            line,
+            level=None,
+            **{
+                it_name: item_des,
+                "Collection": abc.Collection,
+                "iterate": self.resolver.iterate,
+            },
+        )
 
     def _build_path_des(
         self, func: gen.Block, anno_name: str, annotation: "Annotation",
@@ -381,12 +484,15 @@ class DesFactory:
         func.l(f"{self.VNAME} = {anno_name}({self.VNAME})")
 
     def _build_generic_des(
-        self, func: gen.Block, anno_name: str, annotation: "Annotation"
+        self,
+        func: gen.Block,
+        anno_name: str,
+        annotation: "Annotation",
+        namespace: Type = None,
     ):
         serde = annotation.serde
         resolved = annotation.resolved
         self._add_type_check(func, anno_name)
-        self._add_eval(func)
         # Main branch - we have a mapping for a user-defined class.
         # This is where the serde configuration comes in.
         # WINDY PATH AHEAD
@@ -420,7 +526,7 @@ class DesFactory:
                 if serde.fields and len(matched) == len(serde.fields_in):
                     desers = {
                         f: self.resolver._resolve_from_annotation(
-                            serde.fields[f]
+                            serde.fields[f], _namespace=namespace
                         ).transmute
                         for f in matched
                     }
@@ -449,7 +555,9 @@ class DesFactory:
                 translate=self.resolver.translate,
             )
 
-    def _build_des(self, annotation: "Annotation", func_name: str) -> Callable:
+    def _build_des(
+        self, annotation: "Annotation", func_name: str, namespace: Type = None
+    ) -> Callable:
         args = annotation.args
         # Get the "origin" of the annotation.
         # For natives and their typing.* equivs, this will be a builtin type.
@@ -465,11 +573,12 @@ class DesFactory:
         }
         with gen.Block(ns) as main:
             with main.f(func_name, main.param(f"{self.VNAME}")) as func:
-                self._add_vtype(func)
                 if origin not in self.UNRESOLVABLE:
                     self._set_checks(func, anno_name, annotation)
                     if checks.isdatetype(origin):
                         self._build_date_des(func, anno_name, annotation)
+                    elif checks.isuuidtype(origin):
+                        self._build_uuid_des(func, anno_name, annotation)
                     elif origin in {Pattern, re.Pattern}:  # type: ignore
                         self._build_pattern_des(func, anno_name)
                     elif issubclass(origin, pathlib.Path):
@@ -482,18 +591,30 @@ class DesFactory:
                         self._build_builtin_des(func, anno_name, annotation)
                     elif checks.istypeddict(origin):
                         self._build_typeddict_des(
-                            func, anno_name, annotation, total=origin.__total__  # type: ignore
+                            func,
+                            anno_name,
+                            annotation,
+                            total=origin.__total__,  # type: ignore
+                            namespace=namespace,
                         )
                     elif checks.istypedtuple(origin) or checks.isnamedtuple(origin):
-                        self._build_typedtuple_des(func, anno_name, annotation)
+                        self._build_typedtuple_des(
+                            func, anno_name, annotation, namespace=namespace
+                        )
                     elif not args and checks.isbuiltinsubtype(origin):
                         self._build_builtin_des(func, anno_name, annotation)
                     elif checks.ismappingtype(origin):
-                        self._build_mapping_des(func, anno_name, annotation)
+                        self._build_mapping_des(
+                            func, anno_name, annotation, namespace=namespace
+                        )
                     elif checks.iscollectiontype(origin):
-                        self._build_collection_des(func, anno_name, annotation)
+                        self._build_collection_des(
+                            func, anno_name, annotation, namespace=namespace
+                        )
                     else:
-                        self._build_generic_des(func, anno_name, annotation)
+                        self._build_generic_des(
+                            func, anno_name, annotation, namespace=namespace
+                        )
                 func.l(f"{gen.Keyword.RET} {self.VNAME}")
         deserializer = main.compile(ns=ns, name=func_name)
         return deserializer
@@ -563,7 +684,10 @@ class DesFactory:
         return des, validator
 
     def factory(
-        self, annotation: "Annotation", constr: Optional["const.ConstraintsT"] = None
+        self,
+        annotation: "Annotation",
+        constr: Optional["const.ConstraintsT"] = None,
+        namespace: Type = None,
     ) -> Tuple[DeserializerT, "const.ValidatorT"]:
         annotation.serde = annotation.serde or SerdeConfig()
         key = self._get_name(annotation, constr)
@@ -575,7 +699,7 @@ class DesFactory:
                 deserializer = des
                 break
         if not deserializer:
-            deserializer = self._build_des(annotation, key)
+            deserializer = self._build_des(annotation, key, namespace)
 
         deserializer, validator = self._finalize_deserializer(
             annotation, deserializer, constr
