@@ -15,9 +15,11 @@ from typing import (
     cast,
     Union,
     TypeVar,
+    Iterator,
+    Dict,
 )
 
-from typic import checks, constraints as const, util, strict as st
+from typic import checks, constraints as constr, util, strict as st
 from typic.common import (
     EMPTY,
     ORIG_SETTER_NAME,
@@ -28,6 +30,7 @@ from typic.common import (
     Case,
     ReadOnly,
 )
+from typic.compat import ForwardRef
 from typic.strict import StrictModeT
 from .binder import Binder
 from .common import (
@@ -37,11 +40,14 @@ from .common import (
     Annotation,
     SerdeProtocol,
     SerdeProtocolsT,
+    DelayedSerdeProtocol,
+    ForwardDelayedAnnotation,
+    DelayedAnnotation,
 )
 from .des import DesFactory
 from .ser import SerFactory
 from .translator import TranslatorFactory
-
+from ..util import guard_recursion, RecursionDetected
 
 _T = TypeVar("_T")
 
@@ -54,6 +60,7 @@ class Resolver:
         {("asdict", methodcaller("asdict")), ("to_dict", methodcaller("to_dict"))}
     )
     _DYNAMIC = SerFactory._DYNAMIC
+    OPTIONALS = (None, ...)
 
     def __init__(self):
         self.des = DesFactory(self)
@@ -61,10 +68,16 @@ class Resolver:
         self.binder = Binder(self)
         self.translator = TranslatorFactory(self)
         self.bind = self.binder.bind
+        self.__cache = {}
         for typ in checks.STDLIB_TYPES:
             self.resolve(typ)
             self.resolve(Optional[typ])
             self.resolve(typ, is_optional=True)
+            try:
+                self.translator.iterator(typ)
+                self.translator.iterator(typ, values=True)
+            except TypeError:
+                pass
 
     def transmute(self, annotation: Type[ObjectT], value: Any) -> ObjectT:
         """Convert a given value `into` the target annotation.
@@ -107,7 +120,7 @@ class Resolver:
             The higher-order class to translate into.
         """
         resolved: SerdeProtocol = self.resolve(type(value))
-        return resolved.translate(value, target)
+        return resolved.translate(value, target)  # type: ignore
 
     def validate(
         self, annotation: Type[ObjectT], value: Any, *, transmute: bool = False
@@ -129,6 +142,27 @@ class Resolver:
             return resolved.transmute(value)  # type: ignore
         return value
 
+    def iterate(
+        self, obj, *, values: bool = False
+    ) -> Iterator[Union[Tuple[str, Any], Any]]:
+        """Iterate over the fields of an object.
+
+        Parameters
+        ----------
+        obj
+            The object to iterate over
+        values
+            Whether to only yield values of an object's fields. (defaults False)
+        """
+        t = obj.__class__
+        # Extract the type of the enum value if this is an Enum.
+        # Enums classes are iterable and will generate the wrong kind of iterator.
+        if checks.isenumtype(t):
+            obj = obj.value
+            t = obj.__class__
+        iterator = self.translator.iterator(t, values=values)
+        return iterator(obj)
+
     def coerce_value(
         self, value: Any, annotation: Type[ObjectT]
     ) -> ObjectT:  # pragma: nocover
@@ -171,7 +205,7 @@ class Resolver:
                     }
 
                 return SerdeProtocol(
-                    self.annotation(t),
+                    self.annotation(t),  # type: ignore
                     deserializer=None,
                     serializer=serializer,
                     constraints=None,
@@ -287,14 +321,20 @@ class Resolver:
         # Get all the annotated fields
         params = util.safe_get_params(origin)
         # This is probably a builtin and has no signature
-        fields: Mapping[str, Annotation] = {
-            x: self.annotation(
-                y,
+        fields: Dict[
+            str, Union[Annotation, DelayedAnnotation, ForwardDelayedAnnotation]
+        ] = {}
+        for name, anno in util.cached_type_hints(origin).items():
+            kwargs = dict(
                 flags=dataclasses.replace(flags, fields={}),
-                default=getattr(origin, x, EMPTY),
+                default=getattr(origin, name, EMPTY),
+                namespace=origin,
             )
-            for x, y in util.cached_type_hints(origin).items()
-        }
+            with guard_recursion():  # pragma: nocover
+                try:
+                    fields[name] = self.annotation(anno, **kwargs)
+                except RecursionDetected:
+                    fields[name] = self.annotation(anno, recursive=True, **kwargs)
         # Filter out any annotations which aren't part of the object's signature.
         if flags.signature_only:
             fields = {x: fields[x] for x in fields.keys() & params.keys()}
@@ -318,12 +358,22 @@ class Resolver:
             type_omissions = {
                 o for o in omit if checks._type_check(o) or o is NotImplemented
             }
+            type_name_omissions = {util.get_name(o) for o in type_omissions}
             value_omissions = (*(o for o in omit if o not in type_omissions),)
-            fields_out = {
-                x: y
-                for x, y in fields_out.items()
-                if not {fields[x].origin, fields[x].parameter.default} & type_omissions
-            }
+            fields_out_final = {}
+            for name, out in fields_out.items():
+                anno = fields[name]
+                default = anno.parameter.default if anno.parameter else EMPTY
+                if isinstance(anno, ForwardDelayedAnnotation):
+                    if (
+                        not {util.get_name(anno.ref), util.get_name(default)}
+                        & type_name_omissions
+                    ):
+                        fields_out_final[name] = out
+                elif not {anno.origin, default} & type_omissions:
+                    fields_out_final[name] = out
+            fields_out = fields_out_final
+
         fields_in = {y: x for x, y in fields_out.items()}
         if params:
             fields_in = {x: y for x, y in fields_in.items() if y in params}
@@ -349,7 +399,9 @@ class Resolver:
         is_strict: StrictModeT = None,
         flags: "SerdeFlags" = None,
         default: Any = EMPTY,
-    ) -> Annotation:
+        namespace: Type = None,
+        recursive: bool = False,
+    ) -> Union[Annotation, DelayedAnnotation, ForwardDelayedAnnotation]:
         """Get a :py:class:`Annotation` for this type.
 
         Unlike a :py:class:`ResolvedAnnotation`, this does not provide access to a
@@ -374,7 +426,9 @@ class Resolver:
         args = getattr(non_super, "__args__", None)
         # Set whether this is optional/strict
         is_optional = (
-            is_optional or checks.isoptionaltype(non_super) or parameter.default is None
+            is_optional
+            or checks.isoptionaltype(non_super)
+            or parameter.default in self.OPTIONALS
         )
         is_strict = is_strict or checks.isstrict(non_super) or self.STRICT
         is_static = util.origin(use) not in self._DYNAMIC
@@ -395,6 +449,35 @@ class Resolver:
             args = util.get_args(use)
             is_static = util.origin(use) not in self._DYNAMIC
 
+        if use.__class__ is ForwardRef:
+            module, localns = self.__module__, {}
+            if namespace:
+                module = namespace.__module__
+                localns = getattr(namespace, "__dict__", {})
+
+            return ForwardDelayedAnnotation(
+                ref=use,
+                resolver=self,
+                _name=name,
+                parameter=parameter,
+                is_optional=is_optional,
+                is_strict=is_strict,
+                flags=flags,
+                default=default,
+                module=module,
+                localns=localns,
+            )
+        elif use is namespace or recursive:
+            return DelayedAnnotation(
+                type=use,
+                resolver=self,
+                _name=name,
+                parameter=parameter,
+                is_optional=is_optional,
+                is_strict=is_strict,
+                flags=flags,
+                default=default,
+            )
         serde = (
             self._get_configuration(util.origin(use), flags)
             if is_static
@@ -416,26 +499,41 @@ class Resolver:
 
     @functools.lru_cache(maxsize=None)
     def _resolve_from_annotation(
-        self, anno: Annotation, _des: bool = True, _ser: bool = True,
+        self,
+        anno: Union[Annotation, DelayedAnnotation, ForwardDelayedAnnotation],
+        _des: bool = True,
+        _ser: bool = True,
+        _namespace: Type = None,
     ) -> SerdeProtocol:
+        if anno in self.__cache:
+            return self.__cache[anno]
+        if isinstance(anno, (DelayedAnnotation, ForwardDelayedAnnotation)):
+            return DelayedSerdeProtocol(anno)
+
         # FIXME: Simulate legacy behavior. Should add runtime analysis soon (#95)
         if anno.origin is Callable:
             _des, _ser = False, False
         # Build the deserializer
         deserializer, validator, constraints = None, None, None
         if _des:
-            constraints = const.get_constraints(anno.resolved, nullable=anno.optional)
-            deserializer, validator = self.des.factory(anno, constraints)
+            constraints = constr.get_constraints(
+                anno.resolved, nullable=anno.optional, cls=_namespace
+            )
+            deserializer, validator = self.des.factory(
+                anno, constraints, namespace=_namespace
+            )
         # Build the serializer
         serializer: Optional[SerializerT] = self.ser.factory(anno) if _ser else None
         # Put it all together
-        return SerdeProtocol(
+        proto = SerdeProtocol(
             annotation=anno,
             deserializer=deserializer,
             serializer=serializer,
             constraints=constraints,
             validator=validator,
         )
+        self.__cache[anno] = proto
+        return proto
 
     @functools.lru_cache(maxsize=None)
     def resolve(
@@ -447,6 +545,7 @@ class Resolver:
         parameter: Optional[inspect.Parameter] = None,
         is_optional: bool = None,
         is_strict: bool = None,
+        namespace: Type = None,
         _des: bool = True,
         _ser: bool = True,
     ) -> SerdeProtocol:
@@ -492,8 +591,9 @@ class Resolver:
             is_optional=is_optional,
             is_strict=is_strict,
             flags=flags,
+            namespace=namespace,
         )
-        resolved = self._resolve_from_annotation(anno, _des, _ser)
+        resolved = self._resolve_from_annotation(anno, _des, _ser, namespace)
         return resolved
 
     @functools.lru_cache(maxsize=None)
@@ -560,8 +660,9 @@ class Resolver:
                 if field.init is False and util.origin(annotation) is not ReadOnly:
                     annotation = ReadOnly[annotation]  # type: ignore
                 param = param.replace(default=field.default)
-            resolved: SerdeProtocol = self.resolve(
-                annotation, parameter=param, name=name, is_strict=strict
+
+            resolved = self.resolve(
+                annotation, parameter=param, name=name, is_strict=strict, namespace=obj,
             )
             ann[name] = resolved
         try:
