@@ -31,6 +31,7 @@ from typing import (
     KeysView,
     ValuesView,
     Dict,
+    Optional,
 )
 
 from typic import util, checks, gen, types
@@ -40,10 +41,13 @@ from .common import (
     SerializerT,
     SerdeConfig,
     Annotation,
+    ForwardDelayedAnnotation,
     Unprocessed,
     Omit,
     KT,
     VT,
+    DelayedAnnotation,
+    AnnotationT,
 )
 
 
@@ -358,7 +362,7 @@ class SerFactory:
         Iterable_abc,
     )
     _DICTITER = (dict, Mapping, Mapping_abc, MappingProxyType, types.FrozenDict)
-    _PRIMITIVES = (str, int, bool, float, type(None))
+    _PRIMITIVES = (str, int, bool, float, type(None), type(...))
     _DYNAMIC = frozenset(
         {Union, Any, inspect.Parameter.empty, dataclasses.MISSING, ClassVar}
     )
@@ -374,7 +378,7 @@ class SerFactory:
 
     def _check_add_null_check(self, func: gen.Block, annotation: "Annotation"):
         if annotation.optional:
-            with func.b("if o is None:") as b:
+            with func.b(f"if o in {self.resolver.OPTIONALS}:") as b:
                 b.l(f"{gen.Keyword.RET}")
 
     def _add_type_check(self, func: gen.Block, annotation: "Annotation"):
@@ -408,7 +412,7 @@ class SerFactory:
         line = "[*o]"
         ns: Dict[str, Any] = {}
         if annotation.args:
-            arg_a: "Annotation" = self.resolver.annotation(
+            arg_a: "AnnotationT" = self.resolver.annotation(
                 annotation.args[0], flags=annotation.serde.flags
             )
             arg_ser = self.factory(arg_a)
@@ -475,10 +479,10 @@ class SerFactory:
         args = util.get_args(annotation.resolved)
         if args:
             kt, vt = args
-            ktr: "Annotation" = self.resolver.annotation(
+            ktr: "AnnotationT" = self.resolver.annotation(
                 kt, flags=annotation.serde.flags
             )
-            vtr: "Annotation" = self.resolver.annotation(
+            vtr: "AnnotationT" = self.resolver.annotation(
                 vt, flags=annotation.serde.flags
             )
             kser_, vser_ = (self.factory(ktr), self.factory(vtr))
@@ -533,6 +537,8 @@ class SerFactory:
                 self._check_add_null_check(func, annotation)
                 self._add_type_check(func, annotation)
                 line = f"{ser_name}(o)"
+                if annotation.origin in (type(o) for o in self.resolver.OPTIONALS):
+                    line = "None"
                 func.l(f"{gen.Keyword.RET} {line}")
 
         serializer: SerializerT = main.compile(name=func_name, ns=ns)
@@ -582,7 +588,10 @@ class SerFactory:
                 ) as func:
                     self._check_add_null_check(func, annotation)
                     self._add_type_check(func, annotation)
-                    func.l(f"{gen.Keyword.RET} o")
+                    line = "o"
+                    if annotation.origin in (type(o) for o in self.resolver.OPTIONALS):
+                        line = "None"
+                    func.l(f"{gen.Keyword.RET} {line}")
 
             serializer = main.compile(name=func_name, ns=ns)
             self._serializer_cache[func_name] = serializer
@@ -624,6 +633,26 @@ class SerFactory:
             self._serializer_cache[func_name] = serializer
         return serializer
 
-    def factory(self, annotation: "Annotation"):
+    def factory(self, annotation: "AnnotationT"):
+        if isinstance(annotation, (DelayedAnnotation, ForwardDelayedAnnotation)):
+            return DelayedSerializer(annotation, self)
         annotation.serde = annotation.serde or SerdeConfig()
         return self._compile_serializer(annotation)
+
+
+class DelayedSerializer:
+    __slots__ = "anno", "factory", "_serializer"
+
+    def __init__(
+        self,
+        anno: Union["DelayedAnnotation", "ForwardDelayedAnnotation"],
+        factory: SerFactory,
+    ):
+        self.anno = anno
+        self.factory = factory
+        self._serializer: Optional[SerializerT] = None
+
+    def __call__(self, *args, **kwargs):
+        if self._serializer is None:
+            self._serializer = self.factory.factory(self.anno.resolved.annotation)
+        return self._serializer(*args, **kwargs)
