@@ -1,14 +1,16 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 import ast
+import bdb
 import collections
+import contextlib
 import dataclasses
 import functools
 import inspect
 import sys
 from threading import RLock
 from types import MappingProxyType
-from typing import (
+from typing import (  # type: ignore  # ironic...
     Tuple,
     Any,
     Sequence,
@@ -27,10 +29,12 @@ from typing import (
     MutableSet,
     Dict,
     Optional,
+    _eval_type,
 )
 
 import typic.checks as checks
 from typic.ext import json
+from typic.compat import ForwardRef
 
 __all__ = (
     "cached_issubclass",
@@ -43,12 +47,12 @@ __all__ = (
     "filtered_repr",
     "get_args",
     "get_name",
-    "hexhash",
     "origin",
     "resolve_supertype",
     "safe_eval",
     "safe_get_params",
     "simple_attributes",
+    "slotted",
     "typed_dict_signature",
 )
 
@@ -77,10 +81,6 @@ GENERIC_TYPE_MAP = {
 }
 
 
-def hexhash(*args, __order=sys.byteorder, **kwargs) -> str:
-    return hash(f"{args}{kwargs}").to_bytes(8, __order, signed=True).hex()
-
-
 @functools.lru_cache(maxsize=2000, typed=True)
 def safe_eval(string: str) -> Tuple[bool, Any]:
     """Try a few methods to evaluate a string and get the correct Python data-type.
@@ -195,7 +195,7 @@ def get_args(annotation: Any) -> Tuple[Any, ...]:
 
 
 @functools.lru_cache(maxsize=None)
-def get_name(obj: Type) -> str:
+def get_name(obj: Union[Type, ForwardRef]) -> str:
     """Safely retrieve the name of either a standard object or a type annotation.
 
     Examples
@@ -211,7 +211,11 @@ def get_name(obj: Type) -> str:
     'dict'
     """
     if hasattr(obj, "_name") and not hasattr(obj, "__name__"):
-        return obj._name
+        return obj._name  # type: ignore
+    elif isinstance(obj, ForwardRef):
+        return obj.__forward_arg__
+    elif obj in {NotImplemented, None, Ellipsis}:
+        return str(obj)
     return obj.__name__
 
 
@@ -219,7 +223,10 @@ def get_name(obj: Type) -> str:
 def get_qualname(obj: Type) -> str:
     if hasattr(obj, "_name") and not hasattr(obj, "__name__"):
         return repr(obj)
-
+    elif isinstance(obj, ForwardRef):
+        return obj.__forward_arg__
+    elif obj in {NotImplemented, None, Ellipsis}:
+        return str(obj)
     qualname = getattr(obj, "__qualname__", obj.__name__)
     if "<locals>" in qualname:
         return obj.__name__
@@ -404,13 +411,39 @@ def cached_signature(obj: Callable) -> inspect.Signature:
     )
 
 
+def _safe_get_type_hints(annotation: Union[Type, Callable]) -> Dict[str, Type[Any]]:
+    raw_annotations = getattr(annotation, "__annotations__", None) or {}
+    module_name = getattr(annotation, "__module__", None)
+    if module_name:
+        base_globals: Optional[Dict[str, Any]] = sys.modules[module_name].__dict__
+    else:
+        base_globals = None
+    annotations = {}
+    for name, value in raw_annotations.items():
+        if isinstance(value, str):
+            if sys.version_info >= (3, 7):
+                value = ForwardRef(value, is_argument=False)
+            else:
+                value = ForwardRef(value)
+        try:
+            value = _eval_type(value, base_globals, None)
+        except NameError:
+            # this is ok, it can be fixed with update_forward_refs
+            pass
+        annotations[name] = value
+    return annotations
+
+
 @functools.lru_cache(maxsize=None)
-def cached_type_hints(obj: Callable) -> dict:
+def cached_type_hints(obj: Union[Type, Callable]) -> dict:
     """A cached result of :py:func:`typing.get_type_hints`.
 
     We don't want to go through the process of resolving type-hints every time.
     """
-    return get_type_hints(obj)
+    try:
+        return get_type_hints(obj)
+    except NameError:
+        return _safe_get_type_hints(obj)
 
 
 @functools.lru_cache(maxsize=None)
@@ -492,10 +525,56 @@ class TypeMap(Dict[Type, VT]):
         return default
 
 
-def apply_slots(
+class RecursionDetected(RuntimeError):
+    ...
+
+
+class RecursionDetector(bdb.Bdb):  # pragma: nocover
+    """Prevent recursion from even starting.
+
+    https://stackoverflow.com/a/36663046
+
+    Warnings
+    --------
+    While the detector is tracing, no other debug tracers (i.e., codecov!) can trace.
+    """
+
+    def do_clear(self, arg):
+        pass
+
+    def __init__(self, *args):
+        bdb.Bdb.__init__(self, *args)
+        self.stack = set()
+
+    def user_call(self, frame, argument_list):
+        code = frame.f_code
+        if code in self.stack:
+            raise RecursionDetected(f"Caught recursion in: {frame}")
+        self.stack.add(code)
+
+    def user_return(self, frame, return_value):
+        if frame.f_code in self.stack:
+            self.stack.remove(frame.f_code)
+
+
+_detector = RecursionDetector()
+
+
+@contextlib.contextmanager
+def guard_recursion():  # pragma: nocover
+    curtrace = sys.gettrace()
+    _detector.set_trace()
+    try:
+        yield
+    finally:
+        _detector.stack.clear()
+        sys.settrace(curtrace)
+
+
+def slotted(
     _cls: Type = None, *, dict: bool = True, weakref: bool = False,
 ):
-    """Decorator to add __slots__ to class created by dataclass.
+    """Decorator to create a "slotted" version of the provided class.
 
     Returns new class object as it's not possible to add __slots__ after class creation.
 
@@ -538,8 +617,9 @@ def apply_slots(
             cls_dict["__setstate__"] = _slots_setstate
 
         # Prepare new class with slots
-        new_cls = type(cls)(cls.__name__, cls.__bases__, cls_dict)
-        new_cls.__qualname__ = getattr(cls, "__qualname__")
+        new_cls = cls.__class__(cls.__name__, cls.__bases__, cls_dict)
+        new_cls.__qualname__ = cls.__qualname__
+        new_cls.__module__ = cls.__module__
 
         return new_cls
 
